use std::collections::VecDeque;

use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize)]
pub struct History {
    pub entries: VecDeque<String>,
    pub max_size: usize,
    pub current_position: Option<usize>,
}
impl Default for History {
    fn default() -> Self {
        Self {
            entries: Default::default(),
            max_size: 1000,
            current_position: Default::default(),
        }
    }
}

impl History {
    // Find next history that matches a given string from an index
    pub fn search_next(&mut self, _current: &str) -> Option<&str> {
        if let Some(index) = &mut self.current_position {
            if *index < self.entries.len() - 1 {
                *index += 1;
            }
            Some(&self.entries[*index])
        } else if !self.entries.is_empty() {
            self.current_position = Some(0);
            Some(&self.entries[0])
        } else {
            None
        }
    }
    // Find previous history item that matches a given string from an index
    pub fn search_previous(&mut self, _current: &str) -> Option<&str> {
        if let Some(index) = &mut self.current_position {
            if *index == 0 {
                self.current_position = None;
                return Some("");
            }
            *index -= 1;
            Some(&self.entries[*index])
        } else {
            None
        }
    }
}
