use anyhow::anyhow;
use piccolo::{Callback, CallbackReturn, Context, IntoValue, Value};
use web_sys::window;

pub fn storage<'gc>(ctx: Context<'gc>) -> Callback<'gc> {
    Callback::from_fn(&ctx, |ctx, _ex, mut stack| {
        let local_storage = window()
            .ok_or_else(|| anyhow!("Can't get window"))?
            .local_storage()
            .map_err(|e| anyhow!("Local storage error: {}", e.as_string().unwrap_or_default()))?
            .ok_or_else(|| anyhow!("Can't get localstorage"))?;
        let varname: String = stack.from_front(ctx)?;
        let ls_varname: String = format!("userstorage_{}", varname);
        if stack.is_empty() {
            let result = local_storage.get(&ls_varname).map_err(|e| {
                anyhow!("Local storage error: {}", e.as_string().unwrap_or_default())
            })?;
            match result {
                None => stack.push_back(Value::Nil),
                Some(s) => stack.push_back(s.into_value(ctx)),
            }
        } else {
            let value: String = stack.from_front(ctx)?;
            local_storage.set(&ls_varname, &value).map_err(|e| {
                anyhow!("Local storage error: {}", e.as_string().unwrap_or_default())
            })?;
        }
        Ok(CallbackReturn::Return)
    })
}
